///////////////////////////////////////////////////////////////////////////
//
//   Copyright 2004, 2010 Alberto González Palomo
//   Author: Alberto González Palomo - http://sentido-labs.com/
//
//   This file is part of Tutorial Toolkit, abbreviated as TTK.
//
//   TTK is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 3 of the License, or
//   (at your option) any later version.
//
//   TTK is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with TTK; if not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////////////

package com.sentidolabs.ttk;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.Vector;
import java.util.Iterator;

/**
 * Call-out with the appearance of a paper note.
 * The links to the targets of this call-out are drawn as arrows.
 */
public class PaperNoteCallOut extends CallOut
{
    protected Paint arrowStrokePaint;
    protected Stroke arrowStroke;
    protected float arrowStrokeThickness;
    
    public PaperNoteCallOut()
    {
        setStroke(new Color(127, 127, 10), 2.0f);
        setBackground(new Color(255, 255, 80));
        setArrowStroke(new Color(200, 80, 0), 4.0f);
    }
    
    /**
     * Set the stroke for arrow drawing.
     *
     * @param paint colour, texture, etc.
     * @param thickness floating-point number in pixels, e.g. 2.0f
     */
    public void setArrowStroke(Paint paint, float thickness)
    {
        arrowStrokePaint = paint;
        arrowStroke = new BasicStroke(thickness,
                                      BasicStroke.CAP_BUTT,
                                      BasicStroke.JOIN_MITER);
        arrowStrokeThickness = thickness;
    }
    
    public void paintComponent(Graphics g)
    {
        Graphics2D g2 = (Graphics2D) g;
        
        g2.setClip(0, 0, getWidth(), getHeight());
        
        g2.setPaint(arrowStrokePaint);
        g2.setStroke(arrowStroke);
        
        Iterator<Target> targetIterator = targets.iterator();
        while (targetIterator.hasNext()) {
            Line2D line = targetIterator.next().connector;
            drawArrow(line.getX1(), line.getY1(), line.getX2(), line.getY2(), g2);
        }
        
        g2.setPaint(strokePaint);
        g2.setStroke(stroke);
        g2.draw(stroke.createStrokedShape(new Area(getContentBounds())));
    }
    
    private void drawArrow(double xSource, double ySource, double xTarget, double yTarget, Graphics2D g)
    {
        Line2D.Double line = new Line2D.Double();
        GeneralPath path = new GeneralPath();
        double arrowHeadSize;
        arrowHeadSize = 10.0 * arrowStrokeThickness;
        double arrowLength;
        arrowLength = Math.sqrt((xTarget - xSource) * (xTarget - xSource) + (yTarget - ySource) * (yTarget - ySource));
        if (0 == arrowLength) return;
        
        double xArrowheadBase;
        double yArrowheadBase;
        xArrowheadBase = (xSource - xTarget) * arrowHeadSize / arrowLength;
        yArrowheadBase = (ySource - yTarget) * arrowHeadSize / arrowLength;
        line.setLine(xSource, ySource,
                     xTarget + xArrowheadBase, yTarget + yArrowheadBase);
        g.draw(line);
        
        buildTriangle(path,
                      (xTarget + xArrowheadBase - yArrowheadBase / 4.0),
                      (yTarget + yArrowheadBase + xArrowheadBase / 4.0),
                      xTarget, yTarget,
                      (xTarget + xArrowheadBase + yArrowheadBase / 4.0),
                      (yTarget + yArrowheadBase - xArrowheadBase / 4.0));
        g.fill(path);
    }
    
    private void buildTriangle(GeneralPath path, double x0, double y0, double x1, double y1, double x2, double y2)
    {
        path.reset();
        path.moveTo((float) x0, (float) y0);
        path.lineTo((float) x1, (float) y1);
        path.lineTo((float) x2, (float) y2);
        path.closePath();
    }
    
}
